@@ -21,11 +21,7 @@ from .objective import (
     Objective,
     ObjectiveBase,
     OptimizerHistory,
-)
-from .predict import (
-    AmiciPredictor,
-    PredictionConditionResult,
-    PredictionResult,
+    FD,
 )
 from .problem import Problem
 from .result import (
@@ -41,3 +37,5 @@ from . import logging
 from . import startpoint
 from . import store
 from . import visualize
+
+logging.log()
