@@ -9,8 +9,8 @@ import logging
 
 
 def log(name: str = 'pypesto',
-        level: int = logging.DEBUG,
-        console: bool = False,
+        level: int = logging.INFO,
+        console: bool = True,
         filename: str = ''):
     """
     Log messages from a specified name with a specified level to any
@@ -44,7 +44,7 @@ def log(name: str = 'pypesto',
         logger.addHandler(fh)
 
 
-def log_to_console(level: int = logging.DEBUG):
+def log_to_console(level: int = logging.INFO):
     """
     Log to console.
 
@@ -55,7 +55,7 @@ def log_to_console(level: int = logging.DEBUG):
     log(level=level, console=True)
 
 
-def log_to_file(level: int = logging.DEBUG,
+def log_to_file(level: int = logging.INFO,
                 filename: str = '.pypesto_logging.log'):
     """
     Log to file.
