@@ -3,13 +3,14 @@ Objective
 =========
 """
 
-from .amici_calculator import AmiciCalculator
-from .amici import AmiciObjective, AmiciObjectBuilder
+from .base import ObjectiveBase
 from .function import Objective
 from .aggregated import AggregatedObjective
+from .finite_difference import FD
+from .amici_calculator import AmiciCalculator
+from .amici import AmiciObjective, AmiciObjectBuilder
 from .priors import NegLogPriors, NegLogParameterPriors
 from .util import res_to_chi2, sres_to_schi2
-from .base import ObjectiveBase
 from .history import (
     HistoryOptions,
     HistoryBase,
@@ -17,5 +18,6 @@ from .history import (
     MemoryHistory,
     CsvHistory,
     Hdf5History,
-    OptimizerHistory)
+    OptimizerHistory,
+)
 from . import constants
