@@ -3,7 +3,7 @@ import copy
 import tempfile
 import os
 import abc
-from typing import Dict, Sequence, Union, Optional
+from typing import Dict, Optional, Sequence, Tuple, Union
 from collections import OrderedDict
 
 from .base import ObjectiveBase
@@ -50,19 +50,21 @@ class AmiciObjective(ObjectiveBase):
     This class allows to create an objective directly from an amici model.
     """
 
-    def __init__(self,
-                 amici_model: AmiciModel,
-                 amici_solver: AmiciSolver,
-                 edatas: Union[Sequence['amici.ExpData'], 'amici.ExpData'],
-                 max_sensi_order: int = None,
-                 x_ids: Sequence[str] = None,
-                 x_names: Sequence[str] = None,
-                 parameter_mapping: 'ParameterMapping' = None,
-                 guess_steadystate: Optional[bool] = None,
-                 n_threads: int = 1,
-                 fim_for_hess: bool = True,
-                 amici_object_builder: AmiciObjectBuilder = None,
-                 calculator: AmiciCalculator = None):
+    def __init__(
+        self,
+        amici_model: AmiciModel,
+        amici_solver: AmiciSolver,
+        edatas: Union[Sequence['amici.ExpData'], 'amici.ExpData'],
+        max_sensi_order: int = None,
+        x_ids: Sequence[str] = None,
+        x_names: Sequence[str] = None,
+        parameter_mapping: 'ParameterMapping' = None,
+        guess_steadystate: Optional[bool] = None,
+        n_threads: int = 1,
+        fim_for_hess: bool = True,
+        amici_object_builder: AmiciObjectBuilder = None,
+        calculator: AmiciCalculator = None,
+    ):
         """
         Constructor.
 
@@ -245,7 +247,7 @@ class AmiciObjective(ObjectiveBase):
 
         return state
 
-    def __setstate__(self, state: Dict):
+    def __setstate__(self, state: Dict) -> None:
         if state['amici_object_builder'] is None:
             raise NotImplementedError(
                 "AmiciObjective does not support __setstate__ without "
@@ -282,7 +284,11 @@ class AmiciObjective(ObjectiveBase):
 
         self.apply_custom_timepoints()
 
-    def check_sensi_orders(self, sensi_orders, mode) -> bool:
+    def check_sensi_orders(
+        self,
+        sensi_orders: Tuple[int, ...],
+        mode: str,
+    ) -> bool:
         sensi_order = max(sensi_orders)
 
         # dynamically obtain maximum allowed sensitivity order
@@ -300,11 +306,17 @@ class AmiciObjective(ObjectiveBase):
         # evaluate sensitivity order
         return sensi_order <= max_sensi_order
 
-    def check_mode(self, mode):
+    def check_mode(self, mode: str) -> bool:
         return mode in [MODE_FUN, MODE_RES]
 
-    def call_unprocessed(self, x, sensi_orders, mode, edatas=None,
-                         parameter_mapping=None):
+    def call_unprocessed(
+        self,
+        x: np.ndarray,
+        sensi_orders: Tuple[int, ...],
+        mode: str,
+        edatas: Sequence['amici.ExpData'] = None,
+        parameter_mapping: 'ParameterMapping' = None,
+    ):
         sensi_order = max(sensi_orders)
 
         x_dct = self.par_arr_to_dct(x)
@@ -344,7 +356,7 @@ class AmiciObjective(ObjectiveBase):
         """Create dict from parameter vector."""
         return OrderedDict(zip(self.x_ids, x))
 
-    def apply_steadystate_guess(self, condition_ix: int, x_dct: Dict):
+    def apply_steadystate_guess(self, condition_ix: int, x_dct: Dict) -> None:
         """
         Use the stored steadystate as well as the respective  sensitivity (
         if available) and parameter value to approximate the steadystate at
@@ -370,7 +382,11 @@ class AmiciObjective(ObjectiveBase):
         self.edatas[condition_ix].x0 = tuple(x_ss_guess)
 
     def store_steadystate_guess(
-            self, condition_ix: int, x_dct: Dict, rdata: 'amici.ReturnData'):
+        self,
+        condition_ix: int,
+        x_dct: Dict,
+        rdata: 'amici.ReturnData',
+    ) -> None:
         """
         Store condition parameter, steadystate and steadystate sensitivity in
         steadystate_guesses if steadystate guesses are enabled for this
@@ -391,8 +407,8 @@ class AmiciObjective(ObjectiveBase):
         preeq_guesses['x_ss'] = rdata['x_ss']
         preeq_guesses['sx_ss'] = rdata['sx_ss']
 
-    def reset_steadystate_guesses(self):
-        """Resets all steadystate guess data"""
+    def reset_steadystate_guesses(self) -> None:
+        """Reset all steadystate guess data."""
         if not self.guess_steadystate:
             return
 
@@ -400,7 +416,7 @@ class AmiciObjective(ObjectiveBase):
         for condition in self.steadystate_guesses['data']:
             self.steadystate_guesses['data'][condition] = {}
 
-    def apply_custom_timepoints(self):
+    def apply_custom_timepoints(self) -> None:
         """Apply custom timepoints, if applicable.
 
         See the `set_custom_timepoints` method for more information.
@@ -412,9 +428,9 @@ class AmiciObjective(ObjectiveBase):
                 )
 
     def set_custom_timepoints(
-            self,
-            timepoints: Sequence[Sequence[Union[float, int]]] = None,
-            timepoints_global: Sequence[Union[float, int]] = None,
+        self,
+        timepoints: Sequence[Sequence[Union[float, int]]] = None,
+        timepoints_global: Sequence[Union[float, int]] = None,
     ) -> 'AmiciObjective':
         """
         Create a copy of this objective that will be evaluated at custom
